/*********************************************************************
 *
 * Authors: Paolo Badino   - paolo.badino@cern.ch
 *
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  GLite Data Agents Common: MySqlDAOComponent Class
 *
 *  Version info: $Id: MySqlDAOComponent.cpp,v 1.5 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 *********************************************************************/
#include "MySqlDAOComponent.h"
#include "MySqlDAOConfig.h"
#include "glite/data/config/service/ParamValue.hxx"
#include "glite/data/agents/AgentExceptions.h"

#include <glite/data/config/service/Logger.hxx>

#include <time.h>

using namespace glite::data::agents;
using namespace glite::data::agents::dao::mysql;
using namespace glite::config;


/*
 * Constants
 */
namespace {
const char * const MYSQL_DAO_NAME             = "agents-dao-mysql";
                                                                                                                          
// Properties Names
const char * const HOSTNAME_PROPERTY_NAME     = "HostName";
const char * const PORT_PROPERTY_NAME         = "Port";
const char * const DBNAME_PROPERTY_NAME       = "DBName";
const char * const USER_PROPERTY_NAME         = "User";
const char * const PASSWORD_PROPERTY_NAME     = "Password";
const char * const SOCKETNAME_PROPERTY_NAME   = "SocketName";
                                                                                                                          
// Properties Default  Values
const char * const DEFAULT_HOSTNAME           = "localhost";
unsigned int DEFAULT_PORT                     = 0;
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new MySqlDAOComponent();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    MySqlDAOComponent * o = dynamic_cast<MySqlDAOComponent *>(component); 
    if(0 != o){
        delete o;
    }
}
                                                                                
} // End extern "C"

/*
 * MySqlDAOComponent
 *
 * Constructor
 */
MySqlDAOComponent::MySqlDAOComponent():
    ComponentConfiguration(MYSQL_DAO_NAME), 
    m_logger(Category::getInstance(MYSQL_DAO_NAME)){
}

/*
 * ~MySqlDAOComponent
 *
 * Destructor
 */
MySqlDAOComponent::~MySqlDAOComponent(){
}

/*
 * init
 *
 * Initialize the Component
 */
int MySqlDAOComponent::init(const Params& params){
               
    // Better to set the TZ, since it will be used in the conversions
    tzset();
    
    // Get MySqlDAOConfig instance    
    MySqlDAOConfig& dao_config = MySqlDAOConfig::instance();
    try{
        Params::const_iterator it;
        
        // Get DB HostName
        if((it = params.find(HOSTNAME_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),HOSTNAME_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            dao_config.hostName(pv->getValue());
        } else {
            throw ConfigParamException(getName(),HOSTNAME_PROPERTY_NAME, ConfigParamException::MISSING);
        }

        // Get DB Port 
        if((it = params.find(PORT_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),PORT_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            dao_config.port(atol(pv->getValue().c_str()));
        } else {
            dao_config.port(DEFAULT_PORT);
        }
        
        // Get DB Name
        if((it = params.find(DBNAME_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),DBNAME_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            dao_config.dbName(pv->getValue());
        } else {
            throw ConfigParamException(getName(),DBNAME_PROPERTY_NAME, ConfigParamException::MISSING);
        }
        
        // Get DB User
        if((it = params.find(USER_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),USER_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            dao_config.user(pv->getValue());
        } else {
            throw ConfigParamException(getName(),USER_PROPERTY_NAME, ConfigParamException::MISSING);
        }
        
        // Get DB Password
        if((it = params.find(PASSWORD_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),PASSWORD_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            dao_config.password(pv->getValue());
        } else {
            throw ConfigParamException(getName(),PASSWORD_PROPERTY_NAME, ConfigParamException::MISSING);
        }

        // Get DB Socket Name
        if((it = params.find(SOCKETNAME_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),SOCKETNAME_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            dao_config.socketName(pv->getValue());
        }
    
        // Initialize DAO Config
        dao_config.init();
        
        // Log Configuration Values
        log_info(getName() << " Initialized");
    } catch(const LogicError& exc) {
        log_error("Initialization Logic Error for " << getName() << ": " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        log_error("Unexpected Exception during " << getName() << " initialization: " << exc.what());
        throw;
    } catch(...) {
        log_error("Unhandled Exception during " << getName() << " initialization");
        throw;
    }
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int MySqlDAOComponent::config(const Params& params){

    // Nothing to do    
    log_info(getName() << " Configured");
    return 0;
}

/*
 * fini
 *
 * Finalize the Component
 */
int MySqlDAOComponent::fini(){

    // Finalize DAOConfig
    MySqlDAOConfig::instance().fini();
    log_info(getName() << " Finalized");
    
    return 0;
}
