/*********************************************************************
 *
 * Authors: Paolo Badino   - paolo.badino@cern.ch
 *
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  GLite Data Agents Common: MySqlDAOConfig Class
 *
 *  Version info: $Id: MySqlDAOConfig.cpp,v 1.6 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 *********************************************************************/
#include "MySqlDAOConfig.h"

#include <glite/data/config/service/Logger.hxx>
#include <memory>
#include <boost/scoped_ptr.hpp>
#include "glite/data/agents/cred/cred-utility.h"
#include "glite/data/agents/cred/CredServiceException.h"

using namespace glite::data::agents::dao;
using namespace glite::data::agents::dao::mysql;


using glite::data::agents::cred::get_proxy_dn;
using glite::data::agents::cred::CredServiceException;

using boost::scoped_ptr;

/*
 * Constants
 */
namespace {
const char * const MYSQL_DAO_NAME     = "agents-dao-mysql";
    
const char * const SCHEMA_VERS_MAJOR  = "MAJOR";
const char * const SCHEMA_VERS_MINOR  = "MINOR";
const char * const SCHEMA_VERS_PATCH  = "PATCH";
}

/*
 * instance
 *
 * Return the singleton instance
 */
MySqlDAOConfig& MySqlDAOConfig::instance() /*throw ()*/ {
    static MySqlDAOConfig s_instance;
    return s_instance;
}

/*
 * MySqlDAOConfig
 *
 * Constructor
 */
MySqlDAOConfig::MySqlDAOConfig():DAOContextFactory(),
    m_logger(Category::getInstance(MYSQL_DAO_NAME)),
    m_port(0){
}

/*
 * ~MySqlDAOConfig
 *
 * Destructor
 */
MySqlDAOConfig::~MySqlDAOConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
void MySqlDAOConfig::init() /*throw (DAOLogicError)*/{
                                                                                                                          
    try{
        // Log Configuration Values
        m_log_info("Initializing " << MYSQL_DAO_NAME << ". Initialization Parameters are:");
        m_log_info("HostName          : " << m_hostName);
        m_log_info("Port              : " << m_port);
        m_log_info("DBName            : " << m_dbName);
        m_log_info("User              : " << m_user);
        m_log_info("Password          : " << "******");
        m_log_info("SocketName        : " << m_socketName);
        
        // Nothing to Do
        m_log_info("MySQL DAO Initialized");
    
        // get the Agent DN
        try {
            m_agentDn = get_proxy_dn();
            m_log_info("Agent DN is <" << m_agentDn << ">");
        } catch(const CredServiceException& exc){
            m_log_warn("No Agent DN set. Reason is " << exc.what());
        }
    
    } catch(const LogicError& exc) {
        m_log_error("Logic Initialization error: " << exc.what());
        throw DAOLogicError(exc.what());
    } catch(...) {
        m_log_error("Unhandled Exception during MySQL DAO initialization");
        throw;
    }
}

/*
 * fini
 *
 * Finalize the Component
 */
void MySqlDAOConfig::fini(){
    // Nothing to do
}

/*
 * createContext
 *
 * Initialize a MYSQL Context
 */
MySqlDAOContext * MySqlDAOConfig::createContext() /*throw (DAOException)*/{
    // Create the MySql DAO context Object
    std::auto_ptr<MySqlDAOContext> ctx(new MySqlDAOContext(m_agentDn));
    
    // Initialize MYSQL connection Object
    ctx->init();
    
    // Connect To Database
    const char * socket_name = 0;
    if ((0 == m_hostName.compare("localhost")) && (!m_socketName.empty())){
        socket_name = m_socketName.c_str();
    }
    if (0 == mysql_real_connect(&ctx->mySQL(),
                                m_hostName.c_str(),
                                m_user.c_str(),
                                m_password.c_str(),
                                m_dbName.c_str(),
                                m_port,
                                socket_name,
                                CLIENT_FOUND_ROWS)){ // Return the number of found (matched) rows, 
                                                     // not the number of affected rows
        m_log_error("Failed To Connect to the DB: " << mysql_error(&ctx->mySQL()));
        throw DAOException("Connection Failed");
    }
    
    return ctx.release();
}

/*
 * checkSchema
 *
 * Validate the DB schema
 */
void MySqlDAOConfig::checkSchema(DAOContext& daoctx, const std::string& table, unsigned int major,unsigned int minor,unsigned int patch) /*throw (DAOLogicError, DAOException)*/{

    // Initialize MYSQL connection Object
    MySqlDAOContext* ctx = dynamic_cast<MySqlDAOContext*>(&daoctx);
    if(0 == ctx){
        throw DAOLogicError("Invalid DAOContext");
    }
    
    std::string table_escaped;
    ctx->escape(table,table_escaped);
    
    // Query the schema version
    std::stringstream query_stmt;
    query_stmt << "SELECT " 
               << SCHEMA_VERS_MAJOR << ", "
               << SCHEMA_VERS_MINOR << ", "
               << SCHEMA_VERS_PATCH << " "
               << "FROM " << table_escaped << " LIMIT 0,1;";
   
    m_log_trace("Executing Query " << query_stmt.str());
 
    // Get Schema Version
    if(0 != mysql_query(&ctx->mySQL(),query_stmt.str().c_str())){
        m_log_error("Failed To Get Schema Version: " << mysql_error(&ctx->mySQL()));
        m_log_debug("Query was " << query_stmt.str());
        throw DAOException("Query Failed");
    }
    
    MYSQL_RES * result = 0;
    try{
        // Fetch Query Result
        result = mysql_store_result(&ctx->mySQL());
        if(0 == result){
            m_log_error("Failed To Get Result: " << mysql_error(&ctx->mySQL()));
            throw DAOException("Get Schema Version Failed");
        }

        // Get number of Rows
        unsigned int n_rows = mysql_num_rows(result);
        if(0 == n_rows){
            // No Schema Version Stored
            m_log_warn("No Schema Version Stored on the DB. Continue Anyway");
        } else if (1 == n_rows){    
            // Get The number of Fields
            unsigned int n_fields = mysql_num_fields(result);
            if(3 != n_fields){
                m_log_error("Invalid Number of Fields (" << n_fields << "/" << 3 << ") on Result");
                throw DAOException("Get Schema Version Failed");
            }
            // Fetch Row
            MYSQL_ROW row = mysql_fetch_row(result);
            if(0 == row[0]){
                m_log_error("Major Cannot be null");
                throw DAOException("Get Schema Version Failed");
            }
            if(0 == row[1]){
                m_log_error("Minor Cannot be null");
                throw DAOException("Get Schema Version Failed");
            }
            if(0 == row[2]){
                m_log_error("Patch Cannot be null");
                throw DAOException("Get Schema Version Failed");
            }

            // Get Schema Version
            int schema_major = atoi(row[0]);
            int schema_minor = atoi(row[1]);
            int schema_patch = atoi(row[2]);
            
            m_log_debug("Got Version " << schema_major << "." << schema_minor << "." << schema_patch);
            
            // Check if the Version is compatible
            if(!((schema_major == major) && (schema_minor >= minor))){
                m_log_error("Incompatible DB Schema Version: Got " << schema_major << "." << schema_minor << "." << schema_patch << " expected " << major << "." << minor << "." << patch);
                throw DAOException("Incompatible DB Schema Version");
            } else {
                m_log_debug("DB Schema Version is compatible");
            }
        } else {
            m_log_error("Invalid Result. Rows are " << n_rows);
            throw DAOException("Get Schema Version Failed");
        }
    } catch(...){
        if(0 != result){
            mysql_free_result(result);
        }
        throw;
    }
    // Free Result
    mysql_free_result(result);
}
