/*********************************************************************
 *
 * Authors: Paolo Badino   - paolo.badino@cern.ch
 *
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  GLite Data Agents Common: MySqlDAOContext Class
 *
 *  Version info: $Id: MySqlDAOContext.cpp,v 1.7 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 *********************************************************************/
#include "glite/data/agents/dao/mysql/MySqlDAOContext.h"
#include <glite/data/config/service/Logger.hxx>

using namespace glite::data::agents::dao::mysql;

using glite::data::agents::longlong;

/*
 * Constants
 */
namespace {
const char * const MYSQL_DAOCONTEXT_NAME     = "agents-dao-mysql-ctx";
}
/*
 * MySqlDAOContext
 *
 * Constructor
 */
MySqlDAOContext::MySqlDAOContext(const std::string& dn):DAOContext(dn),
    m_logger(Category::getInstance(MYSQL_DAOCONTEXT_NAME)){
}

/*
 * ~MySqlDAOContext
 *
 * Destructor
 */
MySqlDAOContext::~MySqlDAOContext(){
    // Finalize MYSQL connection Object
    mysql_close(&m_mySQL);
}

/*
 * init
 *
 * Initialize the mysql context
 */
void MySqlDAOContext::init() /*throw (DAOLogicError, DAOException)*/{
    // Initialize MYSQL connection Object
    if(0 == mysql_init(&m_mySQL)){
        throw DAOException("Failed to initialize MySQL Context");
    }
}

/*
 * check
 *
 * Check if the Connection is alive
 */
void MySqlDAOContext::check() /*throw (DAOLogicError, DAOException)*/{
    if(0 != mysql_ping(&m_mySQL)){
        throw DAOException("Cannot ping the DB server");
    }
}

    
/*
 * escape
 *
 * Escape a String to prevent SQL injection
 */
void MySqlDAOContext::escape(const std::string& str,std::string& dest){
    char * tmp_str = new char[str.length()*2 + 1];
    mysql_real_escape_string(&m_mySQL, tmp_str, str.c_str(), str.length());
    dest = tmp_str;
    delete [] tmp_str;
}


/*
 * toDatetime
 *
 * Convert a time value into a Datetime string representation
 */
std::string MySqlDAOContext::toDatetime(time_t t){
    
    std::string datetime;
    
    struct tm * tmp_tm = gmtime(&t);
    if(0 != tmp_tm){
        char tmp_str[20];
        if(0 != strftime(tmp_str,20,"%Y-%m-%d %H:%M:%S",tmp_tm)){
            datetime = tmp_str;
        }
    }
    m_log_trace("TimeValue is " << t << " => Datetime is " << datetime);
    return datetime;
}

/*
 * to_timestamp
 *
 * Convert a time value into a Datetime string representation
 */
std::string MySqlDAOContext::toTimestamp(time_t t){
    
    std::string timestamp;
    
    struct tm * tmp_tm = gmtime(&t);
    if(0 != tmp_tm){
        char tmp_str[20];
        if(0 != strftime(tmp_str,20,"%Y%m%d%H%M%S",tmp_tm)){
            timestamp = tmp_str;
        }
    }
    m_log_trace("TimeValue is " << t << " => Timestamp is " << datetime);
    return timestamp;
}

/*
 * toTimeT
 *
 * Convert a Datetime string representation into a time value
 */
time_t MySqlDAOContext::toTimeT(const std::string& timestamp){
    time_t t = 0;
    
    unsigned int year,month,day;
    unsigned int hour,min,sec;
    bool is_timestamp = (timestamp.length() == 14);
    if(true == is_timestamp){
        // Get Timestamp
        if(0 >= sscanf(timestamp.c_str(),"%4d%2d%2d%2d%2d%2d",
                                         &year,&month,&day,&hour,&min,&sec)){
            m_log_error("Cannot Convert Timestamp " << timestamp);
            return (time_t)-1;                                             
        }
    } else {
        // Get Datetime
        if(0 >= sscanf(timestamp.c_str(),"%4d-%2d-%2d %2d:%2d:%2d",
                                         &year,&month,&day,&hour,&min,&sec)){
            m_log_error("Cannot Convert Datetime " << timestamp);
            return (time_t)-1;
        }
    }
        
    struct tm tmp_tm;
    tmp_tm.tm_sec   = sec;
    tmp_tm.tm_min   = min;
    tmp_tm.tm_hour  = hour;
    tmp_tm.tm_mday  = day;
    tmp_tm.tm_mon   = (month > 1)  ?month - 1:0;
    tmp_tm.tm_year  = (year > 1900)?year-1900:0;
    tmp_tm.tm_wday  = 0;
    tmp_tm.tm_yday  = 0;
    // As spotted by D.Smith, the DST shouln't be considered in this 
    // case, since the passed value already consider the offset
    //tmp_tm.tm_isdst = -1;
    tmp_tm.tm_isdst = 0;

    t = mktime(&tmp_tm);
    if((time_t)-1 == t){
        m_log_error("Cannot Convert Datetime " << timestamp);
    } else {
        t -= timezone;
        // Don't consider the timezone (see before)
        //if(1 == tmp_tm.tm_isdst){
        //    t += 3600;
        //}
    }  

    m_log_trace("Timestamp is " << timestamp << " => Time Value is " << t);
    
    return t;
}


/*
 * toLongLong
 *
 * Convert a string representation into a longlong
 */
longlong MySqlDAOContext::toLongLong(const std::string& value){
    longlong l;
#ifdef WIN32
    l = _atoi64(value.c_str());
#else
    l = atoll(value.c_str());
    m_log_trace("String is " << value << " => LongLong is " << l);
#endif    
    return l;
}

/**
 * Convert a longlong into a string representation
 */
std::string MySqlDAOContext::toString(longlong value){
    std::string s;
#ifdef WIN32
    // TODO
#else
    std::stringstream str;
    str << value;
    s = str.str();
    m_log_trace("LongLong is " << value << " => String is " << s);
#endif    
    return s;
}

/*
 * start
 *
 * Start a Transaction
 */
void MySqlDAOContext::start() /*throw (DAOLogicError, DAOException)*/{
    // Set AutoCommit to 0
    const char * query_stmt = "SET AUTOCOMMIT=0;";
    if(0 != mysql_query(&m_mySQL,query_stmt)){
        m_log_error("Failed To Enable Transaction: " << mysql_error(&m_mySQL));
        m_log_debug("Query was " << query_stmt);
        throw DAOException("Enable Transactions Failed");
    }
    // Start Transaction
    query_stmt = "START TRANSACTION;";
    if(0 != mysql_query(&m_mySQL,query_stmt)){
        m_log_error("Failed To Start Transaction: " << mysql_error(&m_mySQL));
        m_log_debug("Query was " << query_stmt);
        throw DAOException("Start Transaction Failed");
    }
        
    m_log_trace("Transaction Started");
}

/*
 * commit
 *
 * Commit a Transaction
 */
void MySqlDAOContext::commit() /*throw (DAOLogicError, DAOException)*/{
    // Commit
    const char * query_stmt = "COMMIT;";
    if(0 != mysql_query(&m_mySQL,query_stmt)){
        m_log_error("Failed To Commit Transaction: " << mysql_error(&m_mySQL));
        m_log_debug("Query was " << query_stmt);
        throw DAOException("Commit Transaction Failed");
    }
    // Set AutoCommit to 1
    query_stmt = "SET AUTOCOMMIT=1;";
    if(0 != mysql_query(&m_mySQL,query_stmt)){
        m_log_error("Failed To Disable Transactions: " << mysql_error(&m_mySQL));
        m_log_debug("Query was " << query_stmt);
        throw DAOException("Disable Transactions Failed");
    }
    m_log_trace("Transaction Committed");
}

/*
 * rollback
 *
 * Rollback a Transaction
 */
void MySqlDAOContext::rollback() /*throw ()*/{
    m_log_info("Rollback Transaction");

    // Rollback
    const char * query_stmt = "ROLLBACK;";
    if(0 != mysql_query(&m_mySQL,query_stmt)){
        m_log_error("Failed To Rollback Transaction: " << mysql_error(&m_mySQL));
        m_log_debug("Query was " << query_stmt);
    } else {
        // Set AutoCommit to 1
        query_stmt = "SET AUTOCOMMIT=1;";
        if(0 != mysql_query(&m_mySQL,query_stmt)){
            m_log_error("Failed To Disable Transactions: " << mysql_error(&m_mySQL));
            m_log_debug("Query was " << query_stmt);
        }
    }
    m_log_debug("Transaction Rolled Back");
}

/*
 * kill
 *
 * kill the context
 */
void MySqlDAOContext::kill() /*throw ()*/{
    m_log_info("Killing context");

    // Get ID of the current connection
    unsigned long id = mysql_thread_id(&m_mySQL);
    
    // kill running query
    mysql_kill(&m_mySQL,id);
    
    // Try to reconnect
    mysql_ping(&m_mySQL);
        
    m_log_info("Context Killed");
}
