/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: MySqlDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: MySqlDaoTestCase.cpp,v 1.3 2007/01/24 12:08:23 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "glite/data/agents/dao/mysql/MySqlDAOContext.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;
using namespace glite::data::agents::dao;
using namespace glite::data::agents::dao::mysql;

using boost::scoped_ptr;

/**
 * CPPUnit MySqlDaoTestCase
 */
class MySqlDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp();

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( MySqlDaoTestCase );
    CPPUNIT_TEST( testQuery_1 );
    CPPUNIT_TEST( testTimeConversion_1 );
    CPPUNIT_TEST( testTimeConversion_2 );
    CPPUNIT_TEST( testTimeConversion_3 );
    CPPUNIT_TEST( testLongLongConversion_1 );
    CPPUNIT_TEST( testLongLongConversion_2 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------
    /**
     * Test a query Method
     */
    void testQuery_1();
    /**
     * Test Time Convertion Method
     */
    void testTimeConversion_1();
    /**
     * Test Time Convertion Method
     */
    void testTimeConversion_2();
    /**
     * Test Time Convertion Method
     */
    void testTimeConversion_3();
    /**
     * Test Time Convertion Method
     */
    void testLongLongConversion_1();
    /**
     * Test Time Convertion Method
     */
    void testLongLongConversion_2();    
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( MySqlDaoTestCase, "MySqlDaoTestCase" );

namespace{
const char * const DUMMY_TABLE          = "t_test_dao_dummy";
const char * const DUMMY_ID             = "dummy_id";
const char * const DUMMY_NAME           = "name";
const char * const DUMMY_COUNTER        = "counter";
const char * const DUMMY_INSERTION_TIME = "insertion_time";
}
/*
 * setUp
 *
 * set-up test case
 */
void MySqlDaoTestCase::setUp(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", dynamic_cast<MySqlDAOContext*>(ctx.get()) != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testQuery_2
 *
 * Test a Query Method
 */
void MySqlDaoTestCase::testQuery_1(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        MySqlDAOContext* mysql_ctx = dynamic_cast<MySqlDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", mysql_ctx != 0);

        // Create Statement
        std::stringstream query_stmt;
        query_stmt << "SELECT " << DUMMY_ID
                       << " , " << DUMMY_NAME
                       << " , " << DUMMY_COUNTER
                       << " , " << DUMMY_INSERTION_TIME
                       << " FROM " << DUMMY_TABLE;
        // Get Schema Version
        if(0 != mysql_query(&mysql_ctx->mySQL(),query_stmt.str().c_str())){
            CPPUNIT_FAIL("Query Failed");
        }
        
        MYSQL_RES * result = 0;
    
        // Fetch Query Result
        result = mysql_store_result(&mysql_ctx->mySQL());
        if(0 == result){
            CPPUNIT_FAIL("Get Result Failed");
        }

        // Get number of Rows
        unsigned int n_rows = mysql_num_rows(result);
        if(0 == n_rows){
            CPPUNIT_FAIL("No Result");
        } else {
            for(int i = 0; i < n_rows; ++i){
                // Get The number of Fields
                unsigned int n_fields = mysql_num_fields(result);
                CPPUNIT_ASSERT_MESSAGE("Invalid number of fields", n_fields == 4);
                
                // Fetch Row
                MYSQL_ROW row = mysql_fetch_row(result);
                // Get ID
                CPPUNIT_ASSERT_MESSAGE("Empty ID", row[0] != 0);
                int id = atoi(row[0]);
                CPPUNIT_ASSERT_MESSAGE("Invalid ID", id != -1);
                // Get Name
                CPPUNIT_ASSERT_MESSAGE("Empty Name", row[1] != 0);
                std::string name = row[1];
                CPPUNIT_ASSERT_MESSAGE("Invalid Name", name.empty() != true);
                // Get Counter
                CPPUNIT_ASSERT_MESSAGE("Empty Counter", row[2] != 0);
                longlong counter = mysql_ctx->toLongLong(row[2]);
                CPPUNIT_ASSERT_MESSAGE("Invalid Counter", counter > 0);
                // Get Insert Time
                CPPUNIT_ASSERT_MESSAGE("Empty Insertion Time", row[3] != 0);
                time_t insertion_time = mysql_ctx->toTimeT(row[3]);
                CPPUNIT_ASSERT_MESSAGE("Invalid Insertion Time", insertion_time != -1);
            }
        }
        mysql_free_result(result);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/**
 * Test Time Convertion Method
 */
void MySqlDaoTestCase::testTimeConversion_1(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        MySqlDAOContext* mysql_ctx = dynamic_cast<MySqlDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", mysql_ctx != 0);

        // Get Current Time
        time_t current;
        time(&current);
        
        // Convert it to Timestamp
        std::string timestamp = mysql_ctx->toTimestamp(current);
        // Convert it to time_t
        time_t time_value = mysql_ctx->toTimeT(timestamp);
        CPPUNIT_ASSERT_MESSAGE("Invalid Time value", time_value != -1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", time_value == current);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/**
 * Test Time Convertion Method
 */
void MySqlDaoTestCase::testTimeConversion_2(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        MySqlDAOContext* mysql_ctx = dynamic_cast<MySqlDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", mysql_ctx != 0);

        // Get Current Time
        time_t current;
        time(&current);
        
        // Convert it to Timestamp
        std::string timestamp = mysql_ctx->toDatetime(current);
        // Convert it to time_t
        time_t time_value = mysql_ctx->toTimeT(timestamp);
        CPPUNIT_ASSERT_MESSAGE("Invalid Time value", time_value != -1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", time_value == current);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/**
 * Test Time Convertion Method
 */
void MySqlDaoTestCase::testTimeConversion_3(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        MySqlDAOContext* mysql_ctx = dynamic_cast<MySqlDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", mysql_ctx != 0);

        // Create Time
        std::string timestamp = "2005-12-15 00:00:00";
        
        // Convert it to Time_t
        time_t time_value = mysql_ctx->toTimeT(timestamp.c_str());
        CPPUNIT_ASSERT_MESSAGE("Invalid Time value", time_value != -1);
        // Convert it to String
        std::string timestamp_2 = mysql_ctx->toDatetime(time_value);
        CPPUNIT_ASSERT_MESSAGE("Invalid Datetime", timestamp_2.empty() == false);
        CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", timestamp_2 == timestamp);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/**
 * Test LongLong Convertion Method
 */
void MySqlDaoTestCase::testLongLongConversion_1(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        MySqlDAOContext* mysql_ctx = dynamic_cast<MySqlDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", mysql_ctx != 0);

        // Get Current Time
        longlong l = 123456789012345ll;
        
        // Convert it to Number
        std::string n = mysql_ctx->toString(l);
        // Convert it to LongLong
        longlong l_2 = mysql_ctx->toLongLong(n);
        CPPUNIT_ASSERT_MESSAGE("Invalid LongLong", l_2 > 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", l_2 == l);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/**
 * Test LongLong Convertion Method
 */
void MySqlDaoTestCase::testLongLongConversion_2(){
    try{
        // Create DAO Context
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        MySqlDAOContext* mysql_ctx = dynamic_cast<MySqlDAOContext*>(ctx.get());
        CPPUNIT_ASSERT_MESSAGE("Invalid DAO Type", mysql_ctx != 0);

        // Get LongLong String
        std::string n = "123456789012345";
        
        // Convert it to LongLong
        longlong l = mysql_ctx->toLongLong(n);
        CPPUNIT_ASSERT_MESSAGE("Invalid LongLong", l > 0);
        
        // Convert it to LongLong String
        std::string n_2 = mysql_ctx->toString(l);
        CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", n_2 == n);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
