/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Agents Common: 17775 Unit Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestCase17775.cpp,v 1.3 2008/11/14 16:04:48 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "glite/data/agents/dao/oracle/StatementPtr.h"
#include "glite/data/agents/dao/oracle/ResultSetPtr.h"
#include "glite/data/agents/dao/mysql/MySqlDAOContext.h"
#include <glite/data/config/service/Logger.hxx>

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents;
using namespace glite::data::agents::dao;


using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::oracle::ResultSetPtr;
using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class TestCase17775 : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    /**
     * tear-down test case
     */
    void tearDown ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TestCase17775 );
    CPPUNIT_TEST( test1 );
    CPPUNIT_TEST( test2 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Conversion Method
     */
    void test1();
    /**
     * Test Read/Write from DB
     */
    void test2();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TestCase17775, "TestCase17775" );

/*
 * Declare Constants. This unit test foresee at beginning
 */
namespace {
}

/*
 * setUp
 *
 * set-up test case
 */
void TestCase17775::setUp(){
    
    try{
         // TODO
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * tearDown
 *
 * tear-down test case
 */
void TestCase17775::tearDown(){
}

/*
 *
 * Test Execute Method
 */
void TestCase17775::test1(){
    
    try{
        Category& logger = Category::getInstance("unit-test-17775_1");
        
        char timebuf[128] = "";
        
        // Get Current Time
        time_t current;
        time(&current);

        // Log Info
        logger.info("Current time is [%d]",current);
        logger.info("Timezone is [%s|%d|%s][%d]",
            tzname[0],(timezone/3600),tzname[1],daylight);

        // Log Local Time
        struct tm local_tm;
        localtime_r(&current, &local_tm);
        asctime_r(&local_tm, timebuf);
        logger.info("Local: %s",timebuf);
        
        // Log GMT Time
        struct tm gmt_tm;
        gmtime_r(&current, &gmt_tm);
        asctime_r(&gmt_tm, timebuf);
        logger.info("GMT: %s",timebuf);
        
        // Get Context per Channel Operations
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        glite::data::agents::dao::oracle::OracleDAOContext* ora_ctx = 
            dynamic_cast<glite::data::agents::dao::oracle::OracleDAOContext*>(ctx.get());
        glite::data::agents::dao::mysql::MySqlDAOContext* mysql_ctx = 
            dynamic_cast<glite::data::agents::dao::mysql::MySqlDAOContext*>(ctx.get());
        if(0 != ora_ctx){
            logger.info("Test the Oracle Conversion");
            
            std::vector<std::string> alter_timezones;
            alter_timezones.push_back("");
            alter_timezones.push_back("+00:00");
            alter_timezones.push_back("+01:00");
            alter_timezones.push_back("+02:00");
            alter_timezones.push_back("+03:00");
            alter_timezones.push_back("+04:00");
            alter_timezones.push_back("+05:00");
            alter_timezones.push_back("+06:00");
            alter_timezones.push_back("+07:00");
            alter_timezones.push_back("+08:00");
            alter_timezones.push_back("+09:00");
            alter_timezones.push_back("+10:00");
            alter_timezones.push_back("+11:00");
            alter_timezones.push_back("-01:00");
            alter_timezones.push_back("-02:00");
            alter_timezones.push_back("-03:00");
            alter_timezones.push_back("-04:00");
            alter_timezones.push_back("-05:00");
            alter_timezones.push_back("-06:00");
            alter_timezones.push_back("-07:00");
            alter_timezones.push_back("-08:00");
            alter_timezones.push_back("-09:00");
            alter_timezones.push_back("-10:00");
            alter_timezones.push_back("-11:00");
            
            std::vector<std::string>::iterator it;
            for(it = alter_timezones.begin(); it != alter_timezones.end(); ++it){
                const std::string& at = (*it);
                if(false == at.empty()){
                    std::string stmt_str;
                    try{
                        // Try to alter the Session timezone
                        StatementPtr stmt(ora_ctx->connection(),ora_ctx->connection().createStatement());
                        // Execute a Query
                        stmt_str = (std::string)"ALTER SESSION SET TIME_ZONE='" + at + "'";
                        stmt->execute(stmt_str);
                        
                        logger.info("Session TimeZone is now '%s'",at.c_str());
                    } catch(const ::oracle::occi::SQLException& exc){
                        logger.error("Got SQL Error <%s>. Query was: [%s]",
                            exc.getMessage().c_str(),stmt_str.c_str());
                        CPPUNIT_FAIL("Can't alter session timezone");
                    }
                }

                // Convert it to Timestamp
                ::oracle::occi::Timestamp timestamp_1 = ora_ctx->toTimestamp(current);
                std::string t1_str = timestamp_1.toText("YYYY,MM,DD HH:MI:SS TZH:TZM",0);
                logger.info("Oracle Timestamp is %s",t1_str.c_str());
                
                // Convert it to time_t
                time_t time_value = ora_ctx->toTimeT(timestamp_1);
                CPPUNIT_ASSERT_MESSAGE("Invalid Time value (1)", time_value != -1);
                CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (1)", time_value == current);
                // Convert it again to a timestamp
                ::oracle::occi::Timestamp timestamp_2 = ora_ctx->toTimestamp(time_value);
                std::string t2_str = timestamp_2.toText("YYYY,MM,DD HH:MI:SS TZH:TZM",0);
                CPPUNIT_ASSERT_MESSAGE("Invalid Convertion        (2)", timestamp_2 == timestamp_1);
                CPPUNIT_ASSERT_MESSAGE("Invalid string Conversion (2)", t1_str == t2_str);
    
                // Convert it again to timestamp
                time_value = ora_ctx->toTimeT(timestamp_2);
                CPPUNIT_ASSERT_MESSAGE("Invalid Time value (3)", time_value != -1);
                CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (3)", time_value == current);
            }                        
        } else if (0 != mysql_ctx){
            // Test the MySql Conversion 
            logger.info("Test the MySql Conversion");
            
            // Convert it to Timestamp
            std::string timestamp_1 = mysql_ctx->toTimestamp(current);
            logger.info("Mysql Timestamp is %s",timestamp_1.c_str());

            // Convert it to time_t
            time_t time_value = mysql_ctx->toTimeT(timestamp_1);
            CPPUNIT_ASSERT_MESSAGE("Invalid Time value", time_value != -1);
            CPPUNIT_ASSERT_MESSAGE("Invalid Conversion", time_value == current);
            
            // Convert it again to a timestamp
            std::string timestamp_2 = mysql_ctx->toTimestamp(time_value);
            CPPUNIT_ASSERT_MESSAGE("Invalid Convertion (2)", timestamp_2 == timestamp_1);

            // Convert it again to timestamp
            time_value = mysql_ctx->toTimeT(timestamp_2);
            CPPUNIT_ASSERT_MESSAGE("Invalid Time value (3)", time_value != -1);
            CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (3)", time_value == current);
        } else {
            CPPUNIT_FAIL("NO DAO Configured");
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 *
 * Test Execute Method
 */
void TestCase17775::test2(){
    
    try{
        Category& logger = Category::getInstance("unit-test-17775_2");
        
        // Get Current Time
        time_t current;
        time(&current);

        // Get Context per Channel Operations
        scoped_ptr<DAOContext> ctx(DAOContextFactory::instance().create());
        glite::data::agents::dao::oracle::OracleDAOContext* ora_ctx = 
            dynamic_cast<glite::data::agents::dao::oracle::OracleDAOContext*>(ctx.get());
        glite::data::agents::dao::mysql::MySqlDAOContext* mysql_ctx = 
            dynamic_cast<glite::data::agents::dao::mysql::MySqlDAOContext*>(ctx.get());
        if(0 != ora_ctx){
            std::vector<std::string> alter_timezones;
            alter_timezones.push_back("");
            alter_timezones.push_back("+00:00");
            alter_timezones.push_back("+01:00");
            alter_timezones.push_back("+02:00");
            alter_timezones.push_back("+03:00");
            alter_timezones.push_back("+04:00");
            alter_timezones.push_back("+05:00");
            alter_timezones.push_back("+06:00");
            alter_timezones.push_back("+07:00");
            alter_timezones.push_back("+08:00");
            alter_timezones.push_back("+09:00");
            alter_timezones.push_back("+10:00");
            alter_timezones.push_back("+11:00");
            alter_timezones.push_back("-01:00");
            alter_timezones.push_back("-02:00");
            alter_timezones.push_back("-03:00");
            alter_timezones.push_back("-04:00");
            alter_timezones.push_back("-05:00");
            alter_timezones.push_back("-06:00");
            alter_timezones.push_back("-07:00");
            alter_timezones.push_back("-08:00");
            alter_timezones.push_back("-09:00");
            alter_timezones.push_back("-10:00");
            alter_timezones.push_back("-11:00");
            
            std::vector<std::string>::iterator it;
            for(it = alter_timezones.begin(); it != alter_timezones.end(); ++it){
                const std::string& at = (*it);
                if(false == at.empty()){
                    std::string stmt_str;
                    try{
                        // Try to alter the Session timezone
                        StatementPtr stmt(ora_ctx->connection(),ora_ctx->connection().createStatement());
                        // Execute a Query
                        stmt_str = (std::string)"ALTER SESSION SET TIME_ZONE='" + at + "'";
                        stmt->execute(stmt_str);
                        
                        logger.info("Session TimeZone is now '%s'",at.c_str());
                    } catch(const ::oracle::occi::SQLException& exc){
                        logger.error("Got SQL Error <%s>. Query was: [%s]",
                            exc.getMessage().c_str(),stmt_str.c_str());
                        CPPUNIT_FAIL("Can't alter session timezone");
                    }
                }
                
                // Create a Statement
                StatementPtr stmt(ora_ctx->connection(),ora_ctx->connection().createStatement());
                // Delete Previous entries
                stmt->setSQL("DELETE FROM test_r_17775");
                stmt->executeUpdate();
                ora_ctx->commit();
                logger.debug("Table cleared");
                
                // Insert an Entry in the table
                stmt->setSQL("INSERT INTO test_r_17775(id,finish_time) VALUES (:1,:2)");
                stmt->setInt(1,0);
                ::oracle::occi::Timestamp timestamp_1 = ora_ctx->toTimestamp(current);
                stmt->setTimestamp(2,timestamp_1);
                stmt->executeUpdate();
                ora_ctx->commit();
                logger.debug("Entry Created");
    
                std::string t1_str = timestamp_1.toText("YYYY,MM,DD HH:MI:SS TZH:TZM",0);
                    
                time_t time_value = -1;
                // Read the Entry
                stmt->setSQL("SELECT finish_time FROM test_r_17775 WHERE id=:1");
                stmt->setInt(1,0);
                {
                ResultSetPtr rs(*stmt,stmt->executeQuery());
                logger.debug("Query Executed");
                if(rs->next()){
                    // Get Data
                    ::oracle::occi::Timestamp timestamp_2 = rs->getTimestamp(1);
                    std::string t2_str = timestamp_2.toText("YYYY,MM,DD HH:MI:SS TZH:TZM",0);
                    CPPUNIT_ASSERT_MESSAGE("Invalid Convertion        (1)", timestamp_2 == timestamp_1);
                    CPPUNIT_ASSERT_MESSAGE("Invalid string Conversion (1)", t1_str == t2_str);
                    // Check Timestamp
                    time_value = ora_ctx->toTimeT(timestamp_2);
                    CPPUNIT_ASSERT_MESSAGE("Invalid Convertion        (2)", time_value == current);
                } else {
                    CPPUNIT_FAIL("NO Data Returned");
                }
                }
                // Update the Entry (with the same value)
                stmt->setSQL("UPDATE test_r_17775 SET finish_time=:1 WHERE id=:2");
                stmt->setInt(2,0);
                ::oracle::occi::Timestamp timestamp_3 = ora_ctx->toTimestamp(time_value);
                stmt->setTimestamp(1,timestamp_3);
                stmt->executeUpdate();
                ora_ctx->commit();
                logger.debug("Entry Updated");
                
                // Read the Entry Again
                time_t time_value_2 = -1;
                // Read the Entry
                stmt->setSQL("SELECT finish_time FROM test_r_17775 WHERE id=:1");
                stmt->setInt(1,0);
                {
                ResultSetPtr rs(*stmt,stmt->executeQuery());
                logger.debug("Query 2 Executed");
                if(rs->next()){
                    // Get Data
                    ::oracle::occi::Timestamp timestamp_3 = rs->getTimestamp(1);
                    std::string t3_str = timestamp_3.toText("YYYY,MM,DD HH:MI:SS TZH:TZM",0);
                    CPPUNIT_ASSERT_MESSAGE("Invalid Convertion        (3)", timestamp_3 == timestamp_1);
                    CPPUNIT_ASSERT_MESSAGE("Invalid string Conversion (3)", t1_str == t3_str);
                    // Check Timestamp
                    time_value_2 = ora_ctx->toTimeT(timestamp_3);
                    CPPUNIT_ASSERT_MESSAGE("Invalid Convertion        (4)", time_value   == current);
                    CPPUNIT_ASSERT_MESSAGE("Invalid Convertion        (5)", time_value_2 == current);
                } else {
                    CPPUNIT_FAIL("NO Data Returned (2)");
                }
                }
            } // end for each session TZ
        } else if(0 != mysql_ctx){
            if(0 != mysql_query(&(mysql_ctx->mySQL()),"DELETE FROM test_r_17775")){
                CPPUNIT_FAIL(mysql_error(&(mysql_ctx->mySQL())));
            }
            mysql_ctx->commit();
            logger.debug("Table cleared");
                
            // Insert an Entry in the table
            std::string timestamp_1 = mysql_ctx->toTimestamp(current);
            std::stringstream query_stmt;
            query_stmt << "INSERT INTO test_r_17775(id,finish_time) VALUES (" 
                       << 0 << " , '" << timestamp_1 << "')"; 
            if(0 != mysql_query(&(mysql_ctx->mySQL()),query_stmt.str().c_str())){
                logger.debug("Query Failed");
                CPPUNIT_FAIL(mysql_error(&(mysql_ctx->mySQL())));
            }
            mysql_ctx->commit();
            logger.debug("Entry Created");
    
            time_t time_value = -1;
            // Read the Entry
            query_stmt.str("");
            query_stmt << "SELECT finish_time FROM test_r_17775 WHERE id=" << 0;
            if(0 != mysql_query(&(mysql_ctx->mySQL()),query_stmt.str().c_str())){
                logger.debug("Query Failed. Was [%s]",query_stmt.str().c_str());
                CPPUNIT_FAIL(mysql_error(&(mysql_ctx->mySQL())));
            }
            MYSQL_RES    * result = 0;
            // Fetch Query Result
            result = mysql_store_result(&(mysql_ctx->mySQL()));
            if(0 == result){
                logger.debug("Store Result Failed");
                CPPUNIT_FAIL(mysql_error(&(mysql_ctx->mySQL())));
            }
            // Get number of Rows
            unsigned int n_rows = mysql_num_rows(result);
            if (n_rows != 1){
                CPPUNIT_FAIL("Invalid number of rows");
            }
            // Get The number of Fields
            unsigned int n_fields = mysql_num_fields(result);
            if(1 != n_fields){
                CPPUNIT_FAIL("Invalid number of fields");
            }
            // Fetch Result
            MYSQL_ROW row = mysql_fetch_row(result);
            if(0 == row){
                CPPUNIT_FAIL("Fetch Row Failed");
            }
            // Check Return Value
            if(0 == row[0]){
                CPPUNIT_FAIL("Invalid Value");
            }
            std::string timestamp_2 = row[0];
            logger.debug("T1 is [%s], T2 is [%s]",timestamp_1.c_str(), timestamp_2.c_str());
            // Check Timestamp
            time_value = mysql_ctx->toTimeT(timestamp_2);
            CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (1)", time_value == current);
            // Free Result
            mysql_free_result(result);
            
            // Update the Entry (with the same value)
            std::string timestamp_3 = mysql_ctx->toTimestamp(time_value);
            query_stmt.str("");
            query_stmt << "UPDATE test_r_17775 SET finish_time='" << timestamp_3
                       << "' WHERE id=" << 0;
            if(0 != mysql_query(&(mysql_ctx->mySQL()),query_stmt.str().c_str())){
                logger.debug("Update Query Failed");
                CPPUNIT_FAIL(mysql_error(&(mysql_ctx->mySQL())));
            }
            mysql_ctx->commit();
            logger.debug("Entry Updated");
            
            // Read the Entry Again
            time_t time_value_2 = -1;
            // Read the Entry
            query_stmt.str("");
            query_stmt << "SELECT finish_time FROM test_r_17775 WHERE id=" << 0;
            if(0 != mysql_query(&(mysql_ctx->mySQL()),query_stmt.str().c_str())){
                logger.debug("Query 2 Failed. Was [%s]",query_stmt.str().c_str());
                CPPUNIT_FAIL(mysql_error(&(mysql_ctx->mySQL())));
            }
            result = 0;
            // Fetch Query Result
            result = mysql_store_result(&(mysql_ctx->mySQL()));
            if(0 == result){
                logger.debug("Store Result 2 Failed");
                CPPUNIT_FAIL(mysql_error(&(mysql_ctx->mySQL())));
            }
            // Get number of Rows
            n_rows = mysql_num_rows(result);
            if (n_rows != 1){
                CPPUNIT_FAIL("Invalid number of rows");
            }
            // Get The number of Fields
            n_fields = mysql_num_fields(result);
            if(1 != n_fields){
                CPPUNIT_FAIL("Invalid number of fields");
            }
            // Fetch Result
            row = mysql_fetch_row(result);
            if(0 == row){
                CPPUNIT_FAIL("Fetch Row Failed");
            }
            // Check Return Value
            if(0 == row[0]){
                CPPUNIT_FAIL("Invalid Value");
            }
            std::string timestamp_4 = row[0];
            logger.debug("T1 is [%s], T4 is [%s]",timestamp_1.c_str(), timestamp_2.c_str());
            // Check Timestamp
            time_value_2 = mysql_ctx->toTimeT(timestamp_4);
            CPPUNIT_ASSERT_MESSAGE("Invalid Conversion (2)", time_value_2 == current);
            // Free Result
            mysql_free_result(result);
        } else {
            CPPUNIT_FAIL("NO DAO Configured");
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
